local type = type
local pairs = pairs


local exactBlacklist = {
	[WorldFrame] = true,
	[ContainerFrame1] = true,
	[ContainerFrame2] = true,
	[ContainerFrame3] = true,
	[ContainerFrame4] = true,
	[ContainerFrame5] = true,
	BattlefieldMinimapTab = true,
	BattlefieldMinimap = true,
	MoneyFuFrame = true,
}

local partialBlacklist = {
	["FuBarPlugin.-Frame"] = true,
	["LibFuBarPlugin%-3%.0_.-_Frame"] = true,
	["FuBarFrame%d"] = true,
}

setfenv(1, Dock.FunctionEnviroment)

function Core:AddPartialNameToBlacklist(name, reason)
	if type(name) == "string" then
		partialBlacklist[name] = type(reason) == "string" and reason or true
	else
		error("bad argument #2 to `AddPartialNameToBlacklist' (string expected, got %s)", type(name))
	end
end



function Core:AddFrameToBlacklist(frame, reason)
	if type(frame) == "string" or type(frame) == "table" and type(frame[0]) == "userdata" then
		exactBlacklist[frame] = type(reason) == "string" and reason or true
	else
		error("bad argument #2 to `AddFrameToBlacklist' (string|frame expected, got %s)", type(frame))
	end
end

function Core:IsFrameBlacklisted(frame)
	local name
	if type(frame) == "string" then
		if exactBlacklist[frame] then
			return true
		end
		name = frame
	elseif type(frame) == "table" and type(frame[0]) == "userdata" then
		if exactBlacklist[frame] then
			return true
		end
		name = frame:GetName()
		if name then
			if exactBlacklist[name] then
				return true
			end
		else
			return
		end
	else
		error("bad argument #2 to `IsFrameBlacklisted' (string|frame expected, got %s)", type(frame))
	end
	for bl, reason in pairs(partialBlacklist) do
		if name:match(bl) then
			exactBlacklist[name] = reason
			return true
		end
	end
end

function Core:GetFrameBlacklistReason(frame)
	local name
	if type(frame) == "string" then
		if exactBlacklist[frame] then
			return type(exactBlacklist[frame]) == "string" and exactBlacklist[frame] or nil
		end
		name = frame
	elseif type(frame) == "table" and type(frame[0]) == "userdata" then
		if exactBlacklist[frame] then
			return type(exactBlacklist[name]) == "string" and exactBlacklist[name] or nil
		end
		name = frame:GetName()
		if name then
			if exactBlacklist[name] then
				return type(exactBlacklist[name]) == "string" and exactBlacklist[name] or nil
			end
		else
			return
		end
	else
		error("bad argument #2 to `GetBlacklistReason' (string|frame expected, got %s)", type(frame))
	end
	for bl, reason in pairs(partialBlacklist) do
		if name:match(bl) then
			exactBlacklist[frame] = reason
			return type(reason) == "string" and reason or nil
		end
	end
end