local type = type
local IsShiftKeyDown = IsShiftKeyDown
local pairs = pairs

setfenv(1, Dock.FunctionEnviroment)

local Core = Core


local DockButtons = DockButtons
local FrameInfo = FrameInfo

local buttonToFrame, buttonToDock = {}, {}


local function OnClick(self, button)
	local dock = Core:GetDockFrame(buttonToDock[self])
	if not dock then --shouldn't happen
		self:Disable()
	else
		dock:SendToDock(buttonToFrame[self], nil, self)
		if type(self.OnSendToDock) == "function" then
			self:OnSendToDock(buttonToFrame[self])
		end
	end
end

local function SetDock(self, dockName, frame)
	frame = frame or self:GetParent()
	if type(dockName) ~= "string" then
		error("bad argument #2 to `SetDock' (string expected, got %s)", type(dockName))
	end
	if type(frame) ~= "table" or type(frame[0]) ~= "userdata" then
		error("bad argument #3 to `SetDock' (frame expected, got %s)", type(frame))
	end
	if not Core:HasDockFrame(dockName) then
		self:Disable()
	else
		self:Enable()
	end
	buttonToFrame[self] = frame
	buttonToDock[self] = dockName
end

local function SetStyle(self, style)
	if type(style) ~= "string" then
		error("bad argument #2 to `SetStyle' (string expected, got %s)", type(style))
	else
		style = style:lower()
		if style ~= "modern" and style ~= "blizzard" then
			error("bad argument #2 to `SetStyle' (modern|blizzard expected, got %s)", style)
		end
	end
	if style == "blizzard" then
		self:SetHighlightTexture[[Interface\Buttons\UI-Panel-MinimizeButton-Highlight]]
		self:SetNormalTexture[[Interface\Addons\Dock\Images\SendToDock.tga]]
		self:GetNormalTexture():SetBlendMode("BLEND")

		self:SetDisabledTexture[[Interface\Addons\Dock\Images\SendToDock.tga]]
		self:GetDisabledTexture():SetDesaturated(true)
		self:GetDisabledTexture():SetBlendMode("BLEND")

		self:SetPushedTexture[[Interface\Addons\Dock\Images\SendToDock.tga]]
		self:GetPushedTexture():SetTexCoord(-.1, 1.1, -.1, 1.1)
	elseif style == "modern" then
		self:SetHighlightTexture[[Interface\Addons\Dock\Images\ModernSendToDock.tga]]
		self:SetNormalTexture[[Interface\Addons\Dock\Images\ModernSendToDock.tga]]
		self:GetNormalTexture():SetBlendMode("ADD")

		self:SetDisabledTexture[[Interface\Addons\Dock\Images\ModernSendToDock.tga]]
		self:GetDisabledTexture():SetDesaturated(true)
		self:GetDisabledTexture():SetBlendMode("ADD")

		self:SetPushedTexture[[Interface\Addons\Dock\Images\ModernSendToDock.tga]]
		self:GetPushedTexture():SetBlendMode("ADD")
		self:GetPushedTexture():SetTexCoord(-.1, 1.1, -.1, 1.1)
	end
end

local function GetFrame(self)
	return buttonToFrame[self]
end

local function GetDock(self)
	return buttonToDock[self]
end

local cache
local function Release(self)
	if not cache then
		cache = {}	
	end
	cache[#cache + 1] = self
	DockButtons[self:GetParent() or ""] = nil
	self:SetParent(nil)
	self:ClearAllPoints()
	self:Hide()
	self.OnNewDock = nil
	self.OnRemoveDock = nil
	self.OnSendToDock = nil
end

function Core:NewDockButton(parent, style)
	if type(parent) ~= "table" or type(parent[0]) ~= "userdata" then
		error("bad argument #2 to `NewDockButton' (frame expected, got %s)", type(parent))
	end
	local f
	if cache then
		f = cache[#cache]
		cache[#cache] = nil
		if #cache == 0 then
			cache = nil
		end
		f:Show()
		f:SetParent(parent)
	else
		f = CreateFrame("Button", nil, parent)
		Core:AddFrameToBlacklist(f, "send to dock buttons can't be docked")
	end

	f:SetHeight(16)
	f:SetWidth(16)

	SetStyle(f, style or "blizzard")
	
	
	f:Disable()

	f:SetScript("OnClick", OnClick)

	f.SetDock = SetDock
	f.Release = Release
	f.SetStyle = SetStyle
	f.GetFrame = GetFrame
	f.GetDock = GetDock

	DockButtons[parent] = f

	return f
end

function Core:GetDockButton(parent)
	return DockButtons[parent]
end